use std::collections::HashMap;

use super::Request;

impl Request {
    /// Extracts the cookies from a Vector and gives back an optional HashMap of Strings
    ///
    /// Returns none if there are no cookies or there is a problem with the cookies, for example
    /// missing a value.
    ///
    /// Removes the `Cookie: ` Header from the Vector of headers
    ///
    /// # Examples
    /// ```
    /// use http::handling::request::Request;
    ///
    ///
    /// let mut request_vec = vec![
    ///     "GET / HTTP/1.1".to_string(),
    ///     "Accept: sdf".to_string(),
    ///     "Cookie: io=23; f=as".to_string(),
    ///     "Format: gzip".to_string(),
    /// ];
    /// let right_finished_vec = vec![
    ///     "GET / HTTP/1.1".to_string(),
    ///     "Accept: sdf".to_string(),
    ///     "Format: gzip".to_string(),
    /// ];
    ///
    /// let mut wrong = vec!["GET / HTTP/1.1".to_string()];
    ///
    /// assert_eq!(None, Request::extract_cookies_from_vec(&mut wrong));
    /// let cookies = Request::extract_cookies_from_vec(&mut request_vec);
    /// assert_eq!(right_finished_vec, request_vec);
    /// assert_eq!("23", cookies.clone().unwrap().get("io").unwrap());
    /// assert_eq!("as", cookies.unwrap().get("f").unwrap());
    /// ```
    ///
    /// # Panics
    /// No Panics
    pub fn extract_cookies_from_vec(headers: &mut Vec<String>) -> Option<HashMap<String, String>> {
        let mut cookies: HashMap<String, String> = HashMap::new();
        let mut cookies_string = if let Some(index) = headers
            .iter()
            .position(|header| header.starts_with("Cookie: "))
        {
            headers.remove(index)
        } else {
            return None;
        };
        cookies_string = cookies_string
            .strip_prefix("Cookie: ")
            .unwrap()
            .trim()
            .trim_matches('"')
            .to_string();
        for cookie in cookies_string.split(';') {
            let Some((name, cookie)) = cookie.split_once('=') else {
                return None;
            };
            cookies
                .entry(name.trim().to_string())
                .or_insert(cookie.trim().to_string());
        }
        Some(cookies)
    }
}

#[cfg(test)]
mod test {
    use crate::handling::request::Request;

    #[test]
    fn test_cookies() {
        let mut request_vec = vec![
            "GET / HTTP/1.1".to_string(),
            "Accept: sdf".to_string(),
            "Cookie: io=23; f=as".to_string(),
            "Format: gzip".to_string(),
        ];
        let right_finished_vec = vec![
            "GET / HTTP/1.1".to_string(),
            "Accept: sdf".to_string(),
            "Format: gzip".to_string(),
        ];

        let mut wrong = vec!["GET / HTTP/1.1".to_string()];

        assert_eq!(None, Request::extract_cookies_from_vec(&mut wrong));
        let cookies = Request::extract_cookies_from_vec(&mut request_vec);
        assert_eq!(right_finished_vec, request_vec);
        assert_eq!("23", cookies.clone().unwrap().get("io").unwrap());
        assert_eq!("as", cookies.unwrap().get("f").unwrap());
    }
}
